@@ -1,13 +1,4 @@
 (**************************************************************)
-(*
- *  Ensemble, 1.10
- *  Copyright 2001 Cornell University, Hebrew University
- *  All rights reserved.
- *
- *  See ensemble/doc/license.txt for further information.
- *)
-(**************************************************************)
-(**************************************************************)
 (* REKEY.ML : switching to a new key using tree structure *)
 (* Authors: Zhen Xiao, Mark Hayden, 6/97 *)
 (* Optimization: Ohad Rodeh 12/97  *)
@@ -155,7 +146,7 @@ let l args vf = Layer.hdr init hdlrs None NoOpt args vf
 		  
 let _ = 
   Param.default "rekey_degree" (Param.Int 2) ;
-  Elink.layer_install name l
+  Layer.install name l
 
 (**************************************************************)
     
