@@ -16,7 +16,7 @@ type bitfield	= int
 type fanout     = int
 type name       = string
 type debug      = string
-type inet       = Hsys.inet
+type inet       = Unix.inet_addr
 type origin     = rank
 type primary    = bool
 type pid        = int
